@@ -1,7 +1,7 @@
 //ff_plain.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2012
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2014
  *
  *  This file is part of RoarAudio PlayList Daemon,
  *  a playlist management daemon for RoarAudio.
@@ -27,12 +27,14 @@
 
 int ff_plain_pl_export(struct fformat_handle * handle, struct roar_vio_calls * vio, struct rpld_playlist  * pl) {
  struct rpld_playlist_entry * cur = rpld_pl_get_first(pl);
+ const char * file;
 
  (void)handle;
 
  while ( cur != NULL ) {
-  if ( cur->io.filename[0] )
-   roar_vio_printf(vio, "%s\n", cur->io.filename);
+  file = rpld_ple_get_filename(cur, -1, NULL);
+  if ( file )
+   roar_vio_printf(vio, "%s\n", file);
   cur = cur->list.next;
  }
 
@@ -63,8 +65,7 @@ int ff_plain_pl_import(struct fformat_handle * handle, struct roar_vio_calls * v
   if ( (plent = rpld_ple_new()) == NULL )
    return -1;
 
-  strncpy(plent->io.filename, buf, sizeof(plent->io.filename));
-  plent->io.filename[sizeof(plent->io.filename)-1] = 0;
+  rpld_ple_push_filename(plent, buf, RPLD_TYPE_UNKNOWN, 1);
 
   rpld_pl_push(pl, plent);
  }
