@@ -1,7 +1,7 @@
 //playback.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2012
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2014
  *
  *  This file is part of RoarAudio PlayList Daemon,
  *  a playlist management daemon for RoarAudio.
@@ -317,8 +317,9 @@ int  playback_del_queue(struct rpld_playlist * pl) {
   return -1;
  }
 
- if ( playback_stop(rpld_pl_get_id(pl), 0, 1) == -1 )
-  return -1;
+ if ( plq->playing )
+  if ( playback_stop(rpld_pl_get_id(pl), 0, 1) == -1 )
+   return -1;
 
  pl->queue = NULL;
 
@@ -366,6 +367,9 @@ int  playback_set_pointer(pli_t queue, int pointer, struct rpld_playlist_pointer
   case POINTER_STARTUP:
     cpointer = &(pl->queue->pointers.startupptr);
    break;
+  case POINTER_FALLBACK:
+    cpointer = &(pl->queue->pointers.fallbackptr);
+   break;
   default:
     rpld_pl_unref(pl);
     roar_err_set(ROAR_ERROR_INVAL);
@@ -409,6 +413,9 @@ struct rpld_playlist_pointer * playback_get_pointer(pli_t queue, int pointer) {
   case POINTER_STARTUP:
     ret = pl->queue->pointers.startupptr;
    break;
+  case POINTER_FALLBACK:
+    ret = pl->queue->pointers.fallbackptr;
+   break;
   default:
     roar_err_set(ROAR_ERROR_INVAL);
    break;
@@ -481,10 +488,13 @@ int  playback_play(pli_t queue) {
  int codec;
  ssize_t magic_len = 0;
  char magic[64];
- char uuid[37];
  const char * content_type;
  struct rpld_queue * plq;
+ const char * filename = NULL;
+ ssize_t filenamecount;
+ ssize_t i;
  int err;
+ int ret;
 
  ROAR_DBG("playback_play(void) = ?");
 
@@ -494,14 +504,18 @@ int  playback_play(pli_t queue) {
  }
 
  if ( (plq = pl->queue) == NULL ) {
+  rpld_pl_unref(pl);
   roar_err_set(ROAR_ERROR_TYPEMM);
   return -1;
  }
 
- if ( plq->playing )
+ if ( plq->playing ) {
+  rpld_pl_unref(pl);
   return 0;
+ }
 
  if ( !(plq->backend->flags & BACKEND_FLAG_CONNECTED) ) {
+  rpld_pl_unref(pl);
   roar_err_set(ROAR_ERROR_NOTCONN);
   return -1;
  }
@@ -510,37 +524,51 @@ int  playback_play(pli_t queue) {
 
  if ( (plent = rpld_pl_get_first(pl)) == NULL ) {
   // retry with autoqueue
-  if ( autoqueue(queue) == -1 )
+  if ( autoqueue(queue) == -1 ) {
+   rpld_pl_unref(pl);
    return -1;
-  if ( (plent = rpld_pl_get_first(pl)) == NULL )
+  }
+  if ( (plent = rpld_pl_get_first(pl)) == NULL ) {
+   rpld_pl_unref(pl);
    return -1;
+  }
  }
 
  ROAR_DBG("playback_play(void) = ?");
 
- if ( *(plent->io.filename) == 0 ) {
-  roar_uuid2str(uuid, plent->uuid, sizeof(uuid));
-  snprintf(plent->io.filename, sizeof(plent->io.filename), "tantalos://%s", uuid);
-  plent->io.filename[sizeof(plent->io.filename)-1] = 0;
- }
+ filenamecount = rpld_ple_num_filename(plent);
+
+ ret = -1;
+ err = ROAR_ERROR_NOENT;
 
  ROAR_DBG("playback_play(void) = ?");
 
- yield_auto(RPLD_YIELD_PAUSE, 0, 0, 0);
+ for (i = 0; ret == -1 && i < filenamecount; i++) {
+  filename = rpld_ple_get_filename(plent, i, NULL);
+  if ( filename == NULL ) {
+   err = roar_error;
+   continue;
+  }
+
+  yield_auto(RPLD_YIELD_PAUSE, 0, 0, 0);
 #if defined(HAVE_FUNC_UALARM) && defined(HAVE_FUNC_SIGINTERRUPT)
- //signal(SIGALRM,  SIG_IGN);
- signal(SIGALRM,  _sighandler_dummy);
- siginterrupt(SIGALRM, 0);
- ualarm(1000*1000, 0);
+  //signal(SIGALRM,  SIG_IGN);
+  signal(SIGALRM,  _sighandler_dummy);
+  siginterrupt(SIGALRM, 0);
+  ualarm(1000*1000, 0);
 #endif
- if ( roar_vio_open_dstr_simple(&(plq->io.file), plent->io.filename, ROAR_VIOF_READ|ROAR_VIOF_NONBLOCK) == -1 ) {
-  ROAR_WARN("playback_play(void): Can not open file: %s: %s", plent->io.filename, roar_error2str(roar_error));
+  ret = roar_vio_open_dstr_simple(&(plq->io.file), filename, ROAR_VIOF_READ|ROAR_VIOF_NONBLOCK);
   err = roar_error;
+
   yield_auto(RPLD_YIELD_UNPAUSE, 0, 0, 0);
+ }
+
+ if ( ret == -1 ) {
+  rpld_pl_unref(pl);
   roar_err_set(err);
+  ROAR_WARN("playback_play(void): Can not open file: %s: %s", filename, roar_error2str(roar_error));
   return -1;
  }
- yield_auto(RPLD_YIELD_UNPAUSE, 0, 0, 0);
 
  ROAR_DBG("playback_play(void) = ?");
 
@@ -548,10 +576,11 @@ int  playback_play(pli_t queue) {
 
  /* TODO: FIXME: do a better check for streams than checking for :// ! */
  // try to guess the type by magic.
- if ( (codec = plent->codec) == -1 && strstr(plent->io.filename, "://") == NULL ) {
+ if ( (codec = plent->codec) == -1 && strstr(filename, "://") == NULL ) {
   if ( (magic_len = roar_vio_read(&(plq->io.file), magic, 64)) < 1 ) {
    err = roar_error;
    roar_vio_close(&(plq->io.file));
+   rpld_pl_unref(pl);
    roar_err_set(err);
    return -1;
   }
@@ -562,6 +591,7 @@ int  playback_play(pli_t queue) {
    if ( roar_vio_lseek(&(plq->io.file), 0, SEEK_SET) == (off_t)-1 ) {
     err = roar_error;
     roar_vio_close(&(plq->io.file));
+    rpld_pl_unref(pl);
     roar_err_set(err);
     return -1;
    }
@@ -609,6 +639,7 @@ int  playback_play(pli_t queue) {
                                       ) == -1 ) {
    err = roar_error;
    roar_vio_close(&(plq->io.file));
+   rpld_pl_unref(pl);
    roar_err_set(err);
    ROAR_DBG("playback_play(void) = -1");
    return -1;
@@ -624,6 +655,7 @@ int  playback_play(pli_t queue) {
    err = roar_error;
    roar_vio_close(&(plq->io.file));
    roar_vio_close(&(plq->io.stream));
+   rpld_pl_unref(pl);
    roar_err_set(err);
    ROAR_DBG("playback_play(void) = -1");
    return -1;
@@ -635,6 +667,7 @@ int  playback_play(pli_t queue) {
     err = roar_error;
     roar_vio_close(&(plq->io.file));
     roar_vio_close(&(plq->io.stream));
+    rpld_pl_unref(pl);
     roar_err_set(err);
     ROAR_DBG("playback_play(void) = -1");
     return -1;
@@ -666,6 +699,8 @@ int  playback_play(pli_t queue) {
  // set volume for next track:
  playback_set_volume(queue);
 
+ rpld_pl_unref(pl);
+
  ROAR_DBG("playback_play(void) = 0");
  return 0;
 }
@@ -683,6 +718,7 @@ int  playback_next(pli_t queue) {
   return -1;
 
  if ( (plq = pl->queue) == NULL ) {
+  rpld_pl_unref(pl);
   roar_err_set(ROAR_ERROR_TYPEMM);
   return -1;
  }
@@ -690,8 +726,10 @@ int  playback_next(pli_t queue) {
  roar_notify_core_emit_simple(RPLD_NOTIFY_PLAYBACK_NEXT, -1, queue, -1, -1, -1, NULL, -1);
 
  was_playing = plq->playing;
- if ( playback_stop(queue, 0, 1) == -1 )
+ if ( playback_stop(queue, 0, 1) == -1 ) {
+  rpld_pl_unref(pl);
   return -1;
+ }
 
  // get next element, if there is none try autoqueue.
  if ( (old = rpld_pl_shift(pl)) == NULL )
@@ -715,6 +753,8 @@ int  playback_next(pli_t queue) {
  if ( was_playing )
   return playback_play(queue);
 
+ rpld_pl_unref(pl);
+
  return 0;
 }
 
@@ -728,6 +768,7 @@ int  playback_prev(pli_t queue) {
   return -1;
 
  if ( (plq = pl->queue) == NULL ) {
+  rpld_pl_unref(pl);
   roar_err_set(ROAR_ERROR_TYPEMM);
   return -1;
  }
@@ -738,16 +779,21 @@ int  playback_prev(pli_t queue) {
 
  old = rpld_pl_shift(plq->history);
 
- if ( old == NULL )
+ if ( old == NULL ) {
+  rpld_pl_unref(pl);
   return -1;
+ }
 
  if ( playback_stop(queue, 0, 1) == -1 ) {
   rpld_pl_add(plq->history, old, -1);
+  rpld_pl_unref(pl);
   return -1;
  }
 
  rpld_pl_add(pl, old, -1);
 
+ rpld_pl_unref(pl);
+  
  if ( was_playing )
   return playback_play(queue);
 
