@@ -1,7 +1,7 @@
      //playlist.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2012
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2014
  *
  *  This file is part of RoarAudio PlayList Daemon,
  *  a playlist management daemon for RoarAudio.
@@ -46,14 +46,18 @@ struct rpld_playlist_entry * rpld_ple_new     (void) {
  plent->global_short_tracknum |= (counter & 0xFFFF) << 8;
  plent->global_short_tracknum |= rand_data ^ RPLD_PL_MAGIC;
 
+ plent->flags                  = RPLD_PLE_FLAG_UNSKIPABLE;
  plent->start                  = -1;
  plent->end                    = -1;
+ plent->wallclock_start        = RPLD_PLE_WALLCLOCK_INVALID;
  plent->rpg.mode               = ROAR_RPGMODE_DEFAULT;
  plent->rpg.mul                =  1;
  plent->rpg.div                =  1;
  plent->codec                  = -1;
+ plent->io.filenames           = NULL;
+ plent->io.num                 = 0;
+ plent->io.slots               = 0;
  plent->meta.genre             = -1;
-
  plent->meta.extra.kv          = NULL;
  plent->meta.extra.kvlen       = -1;
  plent->meta.extra.kv_store    = NULL;
@@ -69,6 +73,8 @@ struct rpld_playlist_entry * rpld_ple_new     (void) {
 }
 
 void                         rpld_ple_free    (struct rpld_playlist_entry * plent) {
+ ssize_t i;
+
  if ( plent == NULL )
   return;
 
@@ -78,16 +84,51 @@ void                         rpld_ple_free    (struct rpld_playlist_entry * plen
  if ( plent->meta.extra.kv != NULL )
   roar_mm_free(plent->meta.extra.kv);
 
+ if ( plent->io.filenames != NULL ) {
+  for (i = 0; i < plent->io.num; i++ )
+   roar_mm_free(plent->io.filenames[i].filename);
+  roar_mm_free(plent->io.filenames);
+ }
+
  memset(plent, 0, sizeof(*plent));
  strncpy(plent->meta.title, "<<<freed>>>", sizeof(plent->meta.title));
 
  roar_mm_free(plent);
 }
 
+static inline struct rpld_filename * __copy_filenames(const struct rpld_filename * src, ssize_t len) {
+ struct rpld_filename * ret;
+ ssize_t i;
+ int err;
+
+ if ( len < 1 )
+  return NULL;
+
+ ret = roar_mm_malloc(sizeof(struct rpld_filename)*len);
+ if ( ret == NULL )
+  return NULL;
+
+ for (i = 0; i < len; i++) {
+  ret[i] = src[i];
+  ret[i].filename = roar_mm_strdup(src[i].filename);
+  if ( ret[i].filename == NULL ) {
+   err = roar_error;
+   for (i--; i >= 0; i--)
+    roar_mm_free(ret[i].filename);
+   roar_mm_free(ret);
+   roar_err_set(err);
+   return NULL;
+  }
+ }
+
+ return ret;
+}
+
 struct rpld_playlist_entry * rpld_ple_copy    (struct rpld_playlist_entry * plent) {
  struct rpld_playlist_entry * ret = NULL;
  ei_t trknl;
  ei_short_t trkns;
+ int err;
 
  if ( plent == NULL )
   return NULL;
@@ -110,6 +151,13 @@ struct rpld_playlist_entry * rpld_ple_copy    (struct rpld_playlist_entry * plen
  if ( ret->meta.extra.kv_store != NULL )
   ret->meta.extra.kvlen = plent->meta.extra.kvlen;
 
+ ret->io.filenames = __copy_filenames(plent->io.filenames, plent->io.num);
+ if ( ret->io.filenames == NULL ) {
+  err = roar_error;
+  rpld_ple_free(ret);
+  roar_err_set(err);
+ }
+
  return ret;
 }
 
@@ -179,6 +227,165 @@ int                          rpld_ple_set_time_hr (struct rpld_playlist_entry *
  return 0;
 }
 
+enum rpld_filename_type      rpld_ple_get_filenametype(const char * type) {
+ const struct {
+  const enum rpld_filename_type type;
+  const char * name;
+ } names[] = {
+  {RPLD_TYPE_ERROR, "error"},
+  {RPLD_TYPE_UNKNOWN, "unknown"},
+  {RPLD_TYPE_AUTOLOCATE, "autolocate"},
+  {RPLD_TYPE_STREAMURL, "streamurl"},
+  {RPLD_TYPE_FILENAME, "filename"},
+  {RPLD_TYPE_FILEURL, "fileurl"}
+ };
+ size_t i;
+
+ if ( type == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return RPLD_TYPE_ERROR;
+ }
+
+ for (i = 0; i < (sizeof(names)/sizeof(*names)); i++) {
+  if ( !!strcasecmp(names[i].name, type) )
+   continue;
+  roar_err_set(ROAR_ERROR_NONE);
+  return names[i].type;
+ }
+
+ roar_err_set(ROAR_ERROR_NOENT);
+ return RPLD_TYPE_ERROR;
+}
+
+int                          rpld_ple_push_filename(struct rpld_playlist_entry * plent, const char * filename, enum rpld_filename_type type, int clear) {
+ const char * tantalos;
+ void * tmp;
+ ssize_t i;
+
+ ROAR_DBG("rpld_ple_push_filename(plent=%p, filename='%s', type=%i, clear=%i) = ?", plent, filename, (int)type, clear);
+
+ if ( plent == NULL || filename == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
+ switch (type) {
+  case RPLD_TYPE_ERROR:
+    roar_err_set(ROAR_ERROR_INVAL);
+    return -1;
+   break;
+  case RPLD_TYPE_UNKNOWN:
+    if ( !strncmp(filename, "tantalos://", 11) ) {
+     type = RPLD_TYPE_AUTOLOCATE;
+    } else if ( !strncmp(filename, "file://", 7) ) {
+     type = RPLD_TYPE_FILENAME;
+    } else if ( !strncmp(filename, "icy://", 6) ) {
+     type = RPLD_TYPE_STREAMURL;
+    }
+   break;
+#ifndef DEBUG
+  default:
+    /* make gcc not warn about missing cases */
+   break;
+#endif
+ }
+
+ tantalos = rpld_ple_get_filename(plent, plent->io.num, NULL);
+ if ( tantalos != NULL ) {
+  if ( !strcmp(tantalos, filename) ) {
+   roar_err_set(ROAR_ERROR_ALREADY);
+   return -1;
+  }
+ }
+
+ for (i = 0; i < plent->io.num; i++) {
+  if ( !strcmp(plent->io.filenames[i].filename, filename) && plent->io.filenames[i].type == type ) {
+   roar_err_set(ROAR_ERROR_ALREADY);
+   return -1;
+  }
+ }
+
+ ROAR_DBG("rpld_ple_push_filename(plent=%p, filename='%s', type=%i, clear=%i) = ?", plent, filename, (int)type, clear);
+
+ if ( plent->io.slots == plent->io.num ) {
+  tmp = roar_mm_realloc(plent->io.filenames, (plent->io.slots + 8)*sizeof(*plent->io.filenames));
+  if ( tmp == NULL )
+   return -1;
+
+  plent->io.slots += 8;
+  plent->io.filenames = tmp;
+ }
+
+ ROAR_DBG("rpld_ple_push_filename(plent=%p, filename='%s', type=%i, clear=%i) = ?", plent, filename, (int)type, clear);
+
+ if ( clear ) {
+  for (i = 0; i < plent->io.num; i++)
+   roar_mm_free(plent->io.filenames[plent->io.num].filename);
+  plent->io.num = 0;
+ }
+
+ plent->io.filenames[plent->io.num].filename = roar_mm_strdup(filename);
+ plent->io.filenames[plent->io.num].type = type;
+
+ plent->io.num++;
+
+ ROAR_DBG("rpld_ple_push_filename(plent=%p, filename='%s', type=%i, clear=%i) = 0", plent, filename, (int)type, clear);
+ return 0;
+}
+
+const char *                 rpld_ple_get_filename(struct rpld_playlist_entry * plent, ssize_t index, enum rpld_filename_type * type) {
+ ssize_t max_index;
+ char uuid[37];
+
+ ROAR_DBG("rpld_ple_get_filename(plent=%p, index=%lli, type=%p) = ?", plent, (long long int)index, type);
+
+ if ( plent == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return NULL;
+ }
+
+ max_index = rpld_ple_num_filename(plent);
+ if ( max_index == -1 )
+  return NULL;
+
+ max_index--;
+
+ if ( index == -1 )
+  index = 0;
+
+ if ( index < 0 ) {
+  roar_err_set(ROAR_ERROR_INVAL);
+  return NULL;
+ } else if ( index == max_index ) {
+  roar_uuid2str(uuid, plent->uuid, sizeof(uuid));
+  snprintf(plent->io.tantalos, sizeof(plent->io.tantalos), "tantalos://%s", uuid);
+  plent->io.tantalos[sizeof(plent->io.tantalos)-1] = 0;
+
+  if ( type != NULL )
+   *type = RPLD_TYPE_AUTOLOCATE;
+
+  return plent->io.tantalos;
+ } else if ( index >= max_index ) {
+  roar_err_set(ROAR_ERROR_NOENT);
+  return NULL;
+ }
+
+ if ( type != NULL )
+  *type = plent->io.filenames[index].type;
+
+ ROAR_DBG("rpld_ple_get_filename(plent=%p, index=%lli, type=%p) = ?", plent, (long long int)index, type);
+ (void)(volatile typeof(plent->io.filenames[index]))plent->io.filenames[index];
+ return plent->io.filenames[index].filename;
+}
+
+ssize_t                      rpld_ple_num_filename(const struct rpld_playlist_entry * plent) {
+ if ( plent == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
+  return -1;
+ }
+
+ return plent->io.num + 1;
+}
 
 static inline int __iscasein(const char * haystack, const char * needle) {
  size_t i, j;
@@ -198,10 +405,11 @@ static inline int __iscasein(const char * haystack, const char * needle) {
  return 0;
 }
 
-int                          rpld_ple_filter  (const struct rpld_playlist_entry * plent, const void * needle, const char * key, int options) {
+int                          rpld_ple_filter  (struct rpld_playlist_entry * plent, const void * needle, const char * key, int options) {
  const char * barn[8];
  const char ** haystacks;
  ssize_t haystackcount = 0;
+ ssize_t filenamecount = -1;
  ssize_t i, j;
  ssize_t needlelen;
  int ret = 0;
@@ -261,13 +469,18 @@ int                          rpld_ple_filter  (const struct rpld_playlist_entry
  // first count how many keys we need to search:
  if ( options & RPLD_PLE_FILTER_NORMKEY ) {
 #define _ck(keyname) if ( key == NULL || !strcasecmp(key, (keyname)) ) { haystackcount++; }
-  _ck("filename")
   _ck("album")
   _ck("title")
   _ck("artist")
   _ck("performer")
   _ck("version")
 #undef _ck
+
+  if ( key == NULL || !strcasecmp(key, "filename") ) {
+   filenamecount = rpld_ple_num_filename(plent);
+   if ( filenamecount > 0 )
+    haystackcount += filenamecount;
+  }
  }
 
  if ( (options & RPLD_PLE_FILTER_EXTKEY) && plent->meta.extra.kv != NULL )
@@ -289,13 +502,17 @@ int                          rpld_ple_filter  (const struct rpld_playlist_entry
 
  if ( options & RPLD_PLE_FILTER_NORMKEY ) {
 #define _ck(keyname,member) if ( key == NULL || !strcasecmp(key, (keyname)) ) { haystacks[j++] = plent->member; }
-  _ck("filename", io.filename)
   _ck("album", meta.album)
   _ck("title", meta.title)
   _ck("artist", meta.artist)
   _ck("performer", meta.performer)
   _ck("version", meta.version)
 #undef _ck
+  if ( key == NULL || !strcasecmp(key, "filename") ) {
+   for (i = 0; i < filenamecount; i++) {
+    haystacks[j++] = rpld_ple_get_filename(plent, i, NULL);
+   }
+  }
  }
 
  if ( (options & RPLD_PLE_FILTER_EXTKEY) && plent->meta.extra.kv != NULL )
@@ -416,10 +633,6 @@ void                         rpld_pl_push     (struct rpld_playlist * pl, struct
  struct rpld_playlist_entry * next = pl->first;
  struct rpld_playlist_entry * last;
 
- if ( rpld_pl_transaction(pl, RPLD_PL_TRANS_ASK) ) {
-  plent->policy = pl->policy;
- }
-
  if ( next == NULL ) {
   pl->first = plent;
  } else {
@@ -439,10 +652,6 @@ void                         rpld_pl_add      (struct rpld_playlist * pl, struct
  struct rpld_playlist_entry * next = pl->first;
  struct rpld_playlist_entry * last;
 
- if ( rpld_pl_transaction(pl, RPLD_PL_TRANS_ASK) ) {
-  plent->policy = pl->policy;
- }
-
  if ( pos == -1 ) {
   plent->list.next = pl->first;
   pl->first        = plent;
@@ -481,8 +690,10 @@ void                         rpld_pl_add      (struct rpld_playlist * pl, struct
 }
 
 struct rpld_playlist_entry * rpld_pl_get_first(struct rpld_playlist * pl) {
- if ( pl == NULL )
+ if ( pl == NULL ) {
+  roar_err_set(ROAR_ERROR_FAULT);
   return NULL;
+ }
 
  if ( pl->flags & RPLD_PL_FLAG_VIRTUAL ) {
   return pl->first->list.next;
@@ -495,6 +706,7 @@ struct rpld_playlist_entry * rpld_pl_shift    (struct rpld_playlist * pl) {
  struct rpld_playlist_entry * first = pl->first;
 
  if ( pl->first == NULL ) {
+  roar_err_set(ROAR_ERROR_NOENT);
   return NULL;
  } else {
   if ( pl->flags & RPLD_PL_FLAG_VIRTUAL ) {
@@ -521,9 +733,11 @@ struct rpld_playlist_entry * rpld_pl_pop      (struct rpld_playlist * pl) {
  if ( pl->flags & RPLD_PL_FLAG_VIRTUAL ) {
   old = cur;
   cur = cur->list.next;
+ }
 
-  if ( cur == NULL )
-   return NULL;
+ if ( cur == NULL ) {
+  roar_err_set(ROAR_ERROR_NOENT);
+  return NULL;
  }
 
  while ((next = cur->list.next) != NULL) {
@@ -775,115 +989,6 @@ int                          rpld_pl_merge    (struct rpld_playlist * pl, struct
  return 0;
 }
 
-int                          rpld_pl_transaction(struct rpld_playlist * pl, int what) {
- if ( pl == NULL )
-  return -1;
-
- switch (what) {
-  case RPLD_PL_TRANS_END:
-    if ( pl->transactions != 1 ) {
-     return -1;
-    }
-    pl->transactions--;
-    if ( pl->transactions != 0 ) {
-     pl->transactions++;
-     return -1;
-    }
-    return 0;
-   break;
-  case RPLD_PL_TRANS_ASK:
-    return pl->transactions;
-   break;
-  case RPLD_PL_TRANS_BEGIN:
-    pl->transactions++;
-    if ( pl->transactions > 1 ) {
-     pl->transactions--;
-     return -1;
-    }
-    return 0;
-   break;
-  default:
-    return -1;
- }
-}
-
-int                          rpld_pl_policy     (struct rpld_playlist * pl, int opolicy, int npolicy) {
- struct rpld_playlist_entry * next;
-
- if ( pl == NULL )
-  return -1;
-
- pl->policy = npolicy;
-
- next = pl->first;
-
- while (next != NULL) {
-  next->policy = opolicy;
-  next = next->list.next;
- }
-
- return 0;
-}
-
-int                          rpld_pl_begin      (struct rpld_playlist * pl, int opolicy, int npolicy) {
- int r = 0;
-
- if ( pl == NULL )
-  return -1;
-
- if ( (r = rpld_pl_transaction(pl, RPLD_PL_TRANS_BEGIN)) != 0 )
-  return r;
-
- if ( (r = rpld_pl_policy(pl, opolicy, npolicy)) != 0 ) {
-  rpld_pl_transaction(pl, RPLD_PL_TRANS_END);
-  return r;
- }
-
- return r;
-}
-
-int                          rpld_pl_commit     (struct rpld_playlist * pl) {
- struct rpld_playlist_entry * cur, * last;
-
- if ( pl == NULL )
-  return -1;
-
- last = NULL;
- cur = pl->first;
-
- if ( pl->flags & RPLD_PL_FLAG_VIRTUAL ) {
-  last = cur;
-  cur  = cur->list.next;
- }
-
- while (cur != NULL) {
-  switch (cur->policy) {
-   case RPLD_PL_POLICY_KEEP:
-     // we ignore this entry, keep it in playlist
-    break;
-   case RPLD_PL_POLICY_DELETE:
-     if ( last == NULL ) {
-      // specal case: we are at begining of playlist
-      pl->first = cur->list.next;
-      rpld_ple_free(cur);
-     } else {
-      // normal case: we are somewhere not the first element
-      last->list.next = cur->list.next;
-      rpld_ple_free(cur);
-     }
-    break;
-   default:
-     ROAR_WARN("rpld_pl_commit(*): Invalid policy type on playlist element. Keeping playlist entry");
-     ROAR_ERR("rpld_pl_commit(*): Incomplet commit on playlist transaction! Check your data!");
-  }
-
-  last = cur;
-  cur  = cur->list.next;
- }
-
- return rpld_pl_transaction(pl, RPLD_PL_TRANS_END);
-}
-
 int                          rpld_pl_set_name (struct rpld_playlist * pl, const char * name) {
  if ( pl == NULL )
   return -1;
@@ -1025,9 +1130,8 @@ int                          rpld_pl_unregister(struct rpld_playlist * pl) {
  if ( g_playlists[index] != pl )
   return -1;
 
- g_playlists[index] = NULL;
-
  rpld_pl_unref(g_playlists[index]);
+ g_playlists[index] = NULL;
 
  return 0;
 }
