@@ -1,7 +1,7 @@
 //plugin.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2012
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2014
  *
  *  This file is part of RoarAudio PlayList Daemon,
  *  a playlist management daemon for RoarAudio.
@@ -99,8 +99,7 @@ static int                         _ple_set_res     (struct rpld_playlist_entry
  if ( file == NULL )
   file = "";
 
- strncpy(plent->io.filename, file, sizeof(plent->io.filename));
- plent->io.filename[sizeof(plent->io.filename)-1] = 0;
+ rpld_ple_push_filename(plent, file, RPLD_TYPE_UNKNOWN, 1);
  plent->codec = codec;
  plent->length = length;
  return 0;
