@@ -1,7 +1,7 @@
 //proto_simple.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2012
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2014
  *
  *  This file is part of RoarAudio PlayList Daemon,
  *  a playlist management daemon for RoarAudio.
@@ -63,6 +63,9 @@ static void remove_spaces(char ** str) {
 
  c = *str;
 
+ if ( c == NULL )
+  return;
+
  for (; *c == ' '; c++);
 
  *str = c;
@@ -177,6 +180,7 @@ static char * proto_simple_help_text[] = {
  "  ADDQUEUE [[TO] " HT_PLI "] [WITH BACKEND backend] [MIXER n] WITH HISTORY " HT_PLI " [WITH ROLE role]\n",
  "  DELQUEUE [" HT_PLI "]\n",
  "  SHOWQUEUE [" HT_PLI "]\n",
+ "  AUTOQUEUE\n",
  "  ADDHISTORY [[TO] " HT_PLI "] [SIZE n]\n",
  "  DELHISTORY [" HT_PLI "]\n",
  "  LISTCLIENTS\n",
@@ -392,6 +396,12 @@ int proto_simple_parse(int client, char * line, size_t len) {
   if ( playback_prev(client_get_queue(client)) != -1 )
    return PROTO_SIMPLE_STATUS_OK;
   return PROTO_SIMPLE_STATUS_ERROR;
+ } else if ( !strcasecmp(cmd, "autoqueue") ) {
+  _CKPERM(ACCLEV_USER);
+  _CKARGS(NULL_OK);
+  if ( autoqueue(client_get_queue(client)) != -1 )
+   return PROTO_SIMPLE_STATUS_OK;
+  return PROTO_SIMPLE_STATUS_ERROR;
 
  } else if ( !strcasecmp(cmd, "pause") ) {
   _CKPERM(ACCLEV_USER);
@@ -986,7 +996,7 @@ int proto_simple_parse(int client, char * line, size_t len) {
  return PROTO_SIMPLE_STATUS_PROERR;
 }
 
-int proto_simple_print_plent(int client, const struct rpld_playlist_entry * plent) {
+int proto_simple_print_plent(int client, struct rpld_playlist_entry * plent) {
  char buffer[1024];
  char uuid[37];
 
@@ -1002,7 +1012,7 @@ int proto_simple_print_plent(int client, const struct rpld_playlist_entry * plen
                         plent->meta.artist,
                         plent->meta.performer,
                         plent->meta.version,
-                        plent->io.filename,
+                        rpld_ple_get_filename(plent, -1, NULL),
                         (__longlong unsigned int)plent->global_tracknum,
                         plent->global_short_tracknum,
                         uuid,
@@ -1704,6 +1714,7 @@ static struct addqueue_para proto_simple_parse_addqueue(int client, char * txt)
 //   ADDQUEUE [[TO] {"Name"|ID}] [WITH BACKEND backend] [MIXER n] WITH HISTORY {"Name"|ID} [WITH ROLE role]
  struct addqueue_para ret = {.queue = -1, .history = -1, .backend = "+default", .mixer = -1, .role = -1};
  struct rpld_playlist * pl;
+ const char * tmp;
 
  ROAR_DBG("proto_simple_parse_addqueue(client=%i, txt='%s') = ?", client, txt);
 
@@ -1761,7 +1772,13 @@ static struct addqueue_para proto_simple_parse_addqueue(int client, char * txt)
    rpld_pl_unref(pl);
   } else if ( !strncasecmp(txt, "with role", 9) ) {
    txt += 9;
-   ret.role = roar_str2role(__parse_string(&txt, "music"));
+   tmp = __parse_string(&txt, "music");
+   if (tmp == NULL) {
+    ret.history = ret.queue = -1;
+    roar_err_set(ROAR_ERROR_ILLSEQ);
+    return ret;
+   }
+   ret.role = roar_str2role(tmp);
   } else {
    ret.history = ret.queue = -1;
    roar_err_set(ROAR_ERROR_ILLSEQ);
@@ -1830,7 +1847,7 @@ static int proto_simple_addqueue(int client, char * txt) {
 }
 
 int proto_simple_listclients (int client) {
- char buf[80];
+ char buf[256];
  int i;
  int proto;
  const char * name;
@@ -1841,11 +1858,18 @@ int proto_simple_listclients (int client) {
    continue;
 
   name = client_get_name(i);
-  if ( name == NULL )
-   name = "Nameless RPLD client";
 
-  snprintf(buf, sizeof(buf), "%3i: [protocol: \"%s\"] \"%s\"\n", i, client_proto2str(proto), name);
+  buf[sizeof(buf)-2] = 0;
+  snprintf(buf, sizeof(buf), "%3i: [protocol: \"%s\", acclev: %s, playlist: %i, queue: %i]%s%s%s\n",
+           i, client_proto2str(proto), client_acclev2str(client_get_acclev(i)),
+           client_get_playlist(i), client_get_queue(i),
+           name == NULL ? "" : " \"", name == NULL ? "" : name, name == NULL ? "" : "\"");
   buf[sizeof(buf)-1] = 0;
+  if ( buf[strlen(buf)-1] != '\n' ) {
+   ROAR_WARN("proto_simple_listclients(client=%i): Buffer too small for client listing: client=%i, proto=%i", client, i, proto);
+   snprintf(buf, sizeof(buf), "%3i: []\n", i);
+   buf[sizeof(buf)-1] = 0;
+  }
   proto_simple_client_puts(client, buf);
  }
 
@@ -1939,7 +1963,7 @@ static int proto_simple_serverinfo(int client) {
 }
 
 static inline void __searchple_handle_playlist(int client, struct rpld_playlist * pl, const void * needle, const char * key, int options, int neg) {
- const struct rpld_playlist_entry * plent = rpld_pl_get_first(pl);
+ struct rpld_playlist_entry * plent = rpld_pl_get_first(pl);
  int ret;
 
  while (plent != NULL) {
@@ -1954,10 +1978,10 @@ static inline void __searchple_handle_playlist(int client, struct rpld_playlist
 static int proto_simple_searchple(int client, char * args) {
  struct rpld_playlist       * pl = NULL;
  const void * needle;
- const char * key, * binkey = NULL;
+ const char * key = NULL, * binkey = NULL;
  int options = RPLD_PLE_FILTER_NORMKEY;
  size_t i;
- char * next;
+ char * next = NULL;
  int bneedle_si;
  discid_t bneedle_discid;
  roar_uuid_t bneedle_uuid;
@@ -2002,8 +2026,11 @@ static int proto_simple_searchple(int client, char * args) {
   return PROTO_SIMPLE_STATUS_ERROR;
  }
 
- if ( binkey != NULL )
+ if ( binkey != NULL ) {
   args = strstr(args, " ");
+  if ( args == NULL )
+   args = "";
+ }
 
  remove_spaces(&args);
  if ( !strncasecmp(args, "case", 4) ) {
@@ -2057,14 +2084,18 @@ static int proto_simple_searchple(int client, char * args) {
   args += 4;
   key = __parse_string(&args, NULL);
   options |= RPLD_PLE_FILTER_EXTKEY;
- } else {
-  key = args;
+ } else if ( !!strcasecmp(args, "from") ){
+  if ( *args ) {
+   key = args;
+  } else {
+   key = NULL;
+  }
  }
 
  if ( binkey != NULL ) {
   if ( key == NULL ) {
    key = binkey;
-  } if ( !!strcasecmp(key, binkey) ) {
+  } else if ( !!strcasecmp(key, binkey) ) {
    roar_err_set(ROAR_ERROR_USER);
    return PROTO_SIMPLE_STATUS_ERROR;
   }
