@@ -1,7 +1,7 @@
 //rpld.c:
 
 /*
- *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2012
+ *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2009-2014
  *
  *  This file is part of RoarAudio PlayList Daemon,
  *  a playlist management daemon for RoarAudio.
@@ -25,6 +25,11 @@
 
 #include "rpld.h"
 
+#if defined(ROAR_HAVE_SETGID) && defined(ROAR_HAVE_SETUID) && defined(ROAR_HAVE_IO_POSIX)
+#include <grp.h>
+#include <pwd.h>
+#endif
+
 #define DEFAULT_ROLE ROAR_ROLE_MUSIC
 
 enum iodir {
@@ -551,6 +556,10 @@ int main (int argc, char * argv[]) {
 
   // --load-playlist is old parameter name and should no longer be used.
   } else if ( !strcmp(k, "--load-playlist") || !strcmp(k, "--playlist-import") || !strcmp(k, "--playlist-export") ) {
+   if ( !strcmp(k, "--load-playlist") ) {
+    roar_debug_option_obsolete(argv[0], k, "--playlist-import", NULL);
+   }
+
    if ( import_num == (sizeof(import_playlist)/sizeof(*import_playlist)) ) {
     ROAR_WARN("Can not import more playlists: maximum number of playlists to import reached.");
     i += 3;
